/**
  ******************************************************************************
  * @file    enc28j60.c
  * @author  YEES
  * @version V1.0.0
  * @date    2011/11/17
  * @brief   ENC28J60. 
  ******************************************************************************
  * @attention
  * (C) COPYRIGHT 2011 YEES 
  ******************************************************************************  
  */ 

#include "lwip/opt.h"
#include "lwip/def.h"
#include "lwip/mem.h"
#include "lwip/pbuf.h"
#include "lwip/sys.h"
#include "lwip/stats.h"
#include "lwip/snmp.h"
#include "netif/etharp.h"
#include "netif/ppp_oe.h"

#include "spi_enc28j60.h"
#include "enc28j60.h"

#include "systick.h"


uint8_t Enc28j60Bank;
uint16_t NextPacketPtr;

void enc28j60CSLow(void)
{
	spiENC28J60_CS_LOW();		//Low
	_delay_us(100);
}

void enc28j60CSHigh(void)
{
	spiENC28J60_CS_HIGH();		//Hi
	_delay_us(100);
}


void nicSetMacAddress(uint8_t* macaddr)
{
	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	enc28j60Write(MAADR5, macaddr[0]);
	enc28j60Write(MAADR4, macaddr[1]);
	enc28j60Write(MAADR3, macaddr[2]);
	enc28j60Write(MAADR2, macaddr[3]);
	enc28j60Write(MAADR1, macaddr[4]);
	enc28j60Write(MAADR0, macaddr[5]);
}


uint8_t enc28j60ReadOp(uint8_t op, uint8_t address)
{
	uint8_t data;
   
	enc28j60CSLow();
	spiENC28J60_SendByte(op | (address & ADDR_MASK));
	_delay_us(100);
	data= spiENC28J60_ReadByte();
	if(address & 0x80)	data= spiENC28J60_ReadByte(); // skip a byte at case of MAC||MII
	_delay_us(100);
	enc28j60CSHigh();

	return data;
}

void enc28j60WriteOp(uint8_t op, uint8_t address, uint8_t data)
{
	enc28j60CSLow();
	spiENC28J60_SendByte(op | (address & ADDR_MASK));
	_delay_us(100);
	spiENC28J60_SendByte(data);
	_delay_us(100);
	enc28j60CSHigh();
}

void enc28j60BeginReadBuffer(void)
{
	spiENC28J60_SendByte(ENC28J60_READ_BUF_MEM);
	_delay_us(100);
}

void enc28j60ReadBuffer(uint16_t len, uint8_t* data)
{
	while(len--)	*data++=spiENC28J60_ReadByte();
}

void enc28j60BeginWriteBuffer(void)
{
	spiENC28J60_SendByte(ENC28J60_WRITE_BUF_MEM);
	_delay_us(100);
}

void enc28j60WriteBuffer(uint16_t len, uint8_t* data)
{
	while(len--)	spiENC28J60_SendByte(*data++);
	_delay_us(100);
}

void enc28j60WriteZeroBuffer(uint16_t len)
{
	while(len--)	spiENC28J60_SendByte(0);
	_delay_us(100);
}


void enc28j60SetBank(uint8_t address)
{
	// set the bank (if needed)
	if((address & BANK_MASK) != Enc28j60Bank)
	{
		// set the bank
		enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, (ECON1_BSEL1|ECON1_BSEL0));
		enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, (address & BANK_MASK)>>5);
		Enc28j60Bank = (address & BANK_MASK);
	}
}

uint8_t enc28j60Read(uint8_t address)
{
	// set the bank
	enc28j60SetBank(address);
	// do the read
	return enc28j60ReadOp(ENC28J60_READ_CTRL_REG, address);
}

void enc28j60Write(uint8_t address, uint8_t data)
{
	// set the bank
	enc28j60SetBank(address);
	// do the write
	enc28j60WriteOp(ENC28J60_WRITE_CTRL_REG, address, data);
}

uint16_t enc28j60PhyRead(uint8_t address)
{
	uint16_t data;

	// Set the right address and start the register read operation
	enc28j60Write(MIREGADR, address);
	enc28j60Write(MICMD, MICMD_MIIRD);

	// wait until the PHY read completes
	while(enc28j60Read(MISTAT) & MISTAT_BUSY);

	// quit reading
	enc28j60Write(MICMD, 0x00);
	
	// get data value
	data  = enc28j60Read(MIRDL);
	data |= enc28j60Read(MIRDH);
	// return the data
	return data;
}

void enc28j60PhyWrite(uint8_t address, uint16_t data)
{
	// set the PHY register address
	enc28j60Write(MIREGADR, address);
	
	// write the PHY data
	enc28j60Write(MIWRL, data);	
	enc28j60Write(MIWRH, data>>8);

	// wait until the PHY write completes
	while(enc28j60Read(MISTAT) & MISTAT_BUSY);
}

void enc28j60Init(uint8_t* macaddr)
{
	spiENC28J60_Init();
		
	spiENC28J60_RESET_LOW();		//Low

	_delay_ms(10);
	spiENC28J60_RESET_HIGH();		//Hi
	_delay_ms(50);
	spiENC28J60_CS_HIGH();			//Hi
	_delay_ms(10);
	
	// perform system reset
	enc28j60WriteOp(ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET);
	
	// check CLKRDY bit to see if reset is complete
	_delay_ms(50);
	while(!(enc28j60Read(ESTAT) & ESTAT_CLKRDY)){
		#ifdef YDEBUG1
		printf("ENC28J60_SOFT_RESET waiting...\n");
		#endif
		;
	}

	// do bank 0 stuff
	// initialize receive buffer
	// 16-bit transfers, must write low byte first
	// set receive buffer start address
	NextPacketPtr = RXSTART_INIT;
	enc28j60Write(ERXSTL, RXSTART_INIT&0xFF);
	enc28j60Write(ERXSTH, RXSTART_INIT>>8);
	// set receive pointer address
	enc28j60Write(ERXRDPTL, RXSTART_INIT&0xFF);
	enc28j60Write(ERXRDPTH, RXSTART_INIT>>8);
	// set receive buffer end
	// ERXND defaults to 0x1FFF (end of ram)
	enc28j60Write(ERXNDL, RXSTOP_INIT&0xFF);
	enc28j60Write(ERXNDH, RXSTOP_INIT>>8);
	// set transmit buffer start
	// ETXST defaults to 0x0000 (beginnging of ram)
	enc28j60Write(ETXSTL, TXSTART_INIT&0xFF);
	enc28j60Write(ETXSTH, TXSTART_INIT>>8);

	// do bank 2 stuff
	// enable MAC receive
	enc28j60Write(MACON1, MACON1_MARXEN|MACON1_TXPAUS|MACON1_RXPAUS);
	// bring MAC out of reset
	enc28j60Write(MACON2, 0x00);
	// enable automatic padding and CRC operations
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, MACON3, MACON3_PADCFG0|MACON3_TXCRCEN|MACON3_FRMLNEN);
//	enc28j60Write(MACON3, MACON3_PADCFG0|MACON3_TXCRCEN|MACON3_FRMLNEN);
	// set inter-frame gap (non-back-to-back)
	enc28j60Write(MAIPGL, 0x12);
	enc28j60Write(MAIPGH, 0x0C);
	// set inter-frame gap (back-to-back)
	enc28j60Write(MABBIPG, 0x12);
	// Set the maximum packet size which the controller will accept
	enc28j60Write(MAMXFLL, MAX_FRAMELEN&0xFF);	
	enc28j60Write(MAMXFLH, MAX_FRAMELEN>>8);

	// do bank 3 stuff
	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	
	printf("\nMAC=%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx\n",
		macaddr[0],macaddr[1],macaddr[2],macaddr[3],macaddr[4],macaddr[5]);

	enc28j60Write(MAADR5, macaddr[0]);
	enc28j60Write(MAADR4, macaddr[1]);
	enc28j60Write(MAADR3, macaddr[2]);
	enc28j60Write(MAADR2, macaddr[3]);
	enc28j60Write(MAADR1, macaddr[4]);
	enc28j60Write(MAADR0, macaddr[5]);
	
	// no loopback of transmitted frames
	enc28j60PhyWrite(PHCON2, PHCON2_HDLDIS);

	// switch to bank 0
	enc28j60SetBank(ECON1);
	// enable interrutps
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, EIE, EIE_INTIE|EIE_PKTIE);
	// enable packet reception
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_RXEN);
/*
	enc28j60PhyWrite(PHLCON, 0x0AA2);

	// setup duplex ----------------------

	// Disable receive logic and abort any packets currently being transmitted
	enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_TXRTS|ECON1_RXEN);
	
	{
		u16 temp;
		// Set the PHY to the proper duplex mode
		temp = enc28j60PhyRead(PHCON1);
//		temp &= ~PHCON1_PDPXMD;
		temp |=  PHCON1_PDPXMD;
		enc28j60PhyWrite(PHCON1, temp);

		// Set the MAC to the proper duplex mode
		temp = enc28j60Read(MACON3);
//		temp &= ~MACON3_FULDPX;
		temp |=  MACON3_FULDPX;
		enc28j60Write(MACON3, temp);
	}

	// Set the back-to-back inter-packet gap time to IEEE specified 
	// requirements.  The meaning of the MABBIPG value changes with the duplex
	// state, so it must be updated in this function.
	// In full duplex, 0x15 represents 9.6us; 0x12 is 9.6us in half duplex
	//enc28j60Write(MABBIPG, DuplexState ? 0x15 : 0x12);	
	
	// Reenable receive logic
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_RXEN);

	// setup duplex ----------------------
*/

	_delay_ms(20);

#ifdef YDEBUG1
	enc28j60RegDump();
#endif

}


void enc28j60ReceiveOverflowRecover(void)
{
	// receive buffer overflow handling procedure

	// recovery completed
}

void enc28j60RegDump(void)
{
	printf("---------------- enc28j60RegDump ---------------- BEGIN\n");
	printf("RevID: 0x%x\n", enc28j60Read(EREVID));

	printf("Ctrl:\n  ");									
	printf("ECON1=%02x ",enc28j60Read(ECON1));				
	printf("ECON2=%02x ",enc28j60Read(ECON2));				
	printf("ESTAT=%02x ",enc28j60Read(ESTAT));				
	printf("EIR=%02x ",  enc28j60Read(EIR));					
	printf("EIE=%02x\n",enc28j60Read(EIE));				

	printf("MAC:\n  ");									
	printf("MACON1=%02x ",enc28j60Read(MACON1));				
	printf("MACON2=%02x ",enc28j60Read(MACON2));				
	printf("MACON3=%02x ",enc28j60Read(MACON3));				
	printf("MACON4=%02x\n  ",enc28j60Read(MACON4));		
	printf("MACADR=%02x:%02x:%02x:%02x:%02x:%02x\n",
		enc28j60Read(MAADR5),enc28j60Read(MAADR4),enc28j60Read(MAADR3),enc28j60Read(MAADR2),
		enc28j60Read(MAADR1),enc28j60Read(MAADR0));					

	printf("Rx:\n  ");																
	printf("ERXST=%x ",enc28j60Read(ERXSTH)*0x100+enc28j60Read(ERXSTL));				
	printf("ERXND=%x ",enc28j60Read(ERXNDH)*0x100+enc28j60Read(ERXNDL));				
	printf("ERXWRPT=%x ",enc28j60Read(ERXWRPTH)*0x100+enc28j60Read(ERXWRPTL));		
	printf("ERXRDPT=%x\n  ",enc28j60Read(ERXRDPTH)*0x100+enc28j60Read(ERXRDPTL));	
	printf("ERXFCON=%x ",enc28j60Read(ERXFCON));										
	printf("EPKTCNT=%x ",enc28j60Read(EPKTCNT));										
	printf("MAMXFL=%x\n",enc28j60Read(MAMXFLH)*0x100+enc28j60Read(MAMXFLL));		

	printf("Tx:\n  ");														
	printf("ETXST=%x ",enc28j60Read(ETXSTH)*0x100+enc28j60Read(ETXSTL));		
	printf("ETXND=%x ",enc28j60Read(ETXNDH)*0x100+enc28j60Read(ETXNDL));		
	printf("MACLCON1=%x ",enc28j60Read(MACLCON1));							
	printf("MACLCON2=%x\n  ",enc28j60Read(MACLCON2));						
	printf("MAPHSUP=%x\n",enc28j60Read(MAPHSUP));							

	printf("---------------- enc28j60RegDump ---------------- END\n");
}


//*******************************************************************************************
// Function : enc28j60_packet_send
// Description : Send packet to network.
//*******************************************************************************************
void enc28j60SendPacket (struct pbuf *p)
{
	struct pbuf *q;
	uint16_t totallen, padlen;
	#ifdef YDEBUG2
	uint16_t i, j;
	#endif
  

   	totallen = p->tot_len;

	if (totallen < 60)
	{
	 	padlen = 60 - totallen;
	 	totallen = 60;
	} 
	else
	{
		padlen = 0;
	}
	
	#ifdef YDEBUG1
	printf("enc28j60SendPacket length=%hu \n", totallen);
	#endif

	//Errata: Transmit Logic reset
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRST);
	enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_TXRST);

	// Set the write pointer to start of transmit buffer area
	enc28j60Write(EWRPTL, TXSTART_INIT & 0xff);
	enc28j60Write(EWRPTH, TXSTART_INIT>>8);
	// Set the TXND pointer to correspond to the packet size given
	enc28j60Write(ETXNDL, (TXSTART_INIT+totallen));
	enc28j60Write(ETXNDH, (TXSTART_INIT+totallen)>>8);
	
	// write per-packet control byte
	enc28j60WriteOp(ENC28J60_WRITE_BUF_MEM, 0, 0x00);

	enc28j60CSLow();

	enc28j60BeginWriteBuffer();

	#ifdef YDEBUG2
	j = 0;
	#endif
	for(q = p; q != NULL; q = q->next) 
	{
		enc28j60WriteBuffer(q->len, q->payload);
		//@
		#ifdef YDEBUG2
		for(i=0;i<q->len;i++){
			printf("%02X ", ((uint8_t*)(q->payload))[i]);
			j++;
			if(j%16==0){
				printf("\n");
			}
		}
		#endif
		//@
	}
	#ifdef YDEBUG2
	if (padlen==0){
		if(j%16!=0)printf("\n");
	}
	#endif

	if (padlen)
	{
		enc28j60WriteZeroBuffer(padlen);
		//@
		#ifdef YDEBUG2
		for(i=0;i<padlen;i++){
			printf("%02X ", 0x00);
			j++;
			if(j%16==0){
				printf("\n");
			}
		}
		if(j%16!=0)printf("\n");
		#endif
		//@
	}
	enc28j60CSHigh();
		
	// send the contents of the transmit buffer onto the network
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRTS);
	
        // Reset the transmit logic problem. See Rev. B4 Silicon Errata point 12.
	if( (enc28j60Read(EIR) & EIR_TXERIF) ){
		enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_TXRTS);
    }
}


//*******************************************************************************************
// Function : enc28j60_getpacket
// Description : Get packet from network.
//*******************************************************************************************
struct pbuf* enc28j60GetPacket (struct netif *netif)
{
	struct pbuf *p, *q;
	uint16_t rxstat;
	uint16_t len;
	#ifdef YDEBUG2
	uint16_t i, j;
	#endif
	
	// check if a packet has been received and buffered
	if( !(enc28j60Read(EIR) & EIR_PKTIF) )
		return NULL;
	
	// Make absolutely certain that any previous packet was discarded	
	//if( WasDiscarded == FALSE)
	//	MACDiscardRx();

	// Set the read pointer to the start of the received packet
	enc28j60Write(ERDPTL, (NextPacketPtr));
	enc28j60Write(ERDPTH, (NextPacketPtr)>>8);
	// read the next packet pointer
	NextPacketPtr  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	NextPacketPtr |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the packet length
	len  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	len |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the receive status
	rxstat  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	rxstat |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
		
	#ifdef YDEBUG1
	printf("enc28j60GetPacket length=%hu \n", len);
	#endif
	
	// limit retrieve length
	// (we reduce the MAC-reported length by 4 to remove the CRC)
	if(len > MAX_FRAMELEN){
		len = MAX_FRAMELEN;
	}
	#if ETH_PAD_SIZE
	if(len > (MAX_FRAMELEN - ETH_PAD_SIZE)){
		len = MAX_FRAMELEN - ETH_PAD_SIZE;
	}
	#endif
		  
	#if ETH_PAD_SIZE
		len += ETH_PAD_SIZE; /* allow room for Ethernet padding */
	#endif

	 /* We allocate a pbuf chain of pbufs from the pool. */
	 p = pbuf_alloc(PBUF_RAW, len, PBUF_POOL);		
	  	
	if(p != NULL){		
		enc28j60CSLow();
		
		// issue read command
		enc28j60BeginReadBuffer();
			
		/* We iterate over the pbuf chain until we have read the entire
		 * packet into the pbuf. */
		for (q = p; q != NULL; q = q->next) {			
			// check CRC and symbol errors (see datasheet page 44, table 7-3):
			// The ERXFCON.CRCEN is set by default. Normally we should not
			// need to check this.
			if ((rxstat & 0x80)==0){
                // invalid
                len=0;
			}else{
                // copy the packet from the receive buffer
                enc28j60ReadBuffer(q->len, q->payload);
			}
		}
		enc28j60CSHigh();

		//@
		#ifdef YDEBUG2
		j = 0;
		for (q = p; q != NULL; q = q->next) {
			for(i=0;i<q->len;i++){
				printf("%02X ", ((unsigned char*)(q->payload))[i]);
				j++;
				if(j%16==0){
					printf("\n");
				}
			}
		}
		if(j%16!=0)printf("\n");
		#endif
			    		
			
		#if ETH_PAD_SIZE
			pbuf_header(p, ETH_PAD_SIZE); /* reclaim the padding word */
		#endif

		LINK_STATS_INC(link.recv);	
		
	}else{
		#ifdef YDEBUG1
		printf("enc28j60GetPacket pbuf_alloc ERROR\n");
		#endif
		return NULL;
	}				
	
	// Move the RX read pointer to the start of the next received packet
	// This frees the memory we just read out
	enc28j60Write(ERXRDPTL, (NextPacketPtr));
	enc28j60Write(ERXRDPTH, (NextPacketPtr)>>8);

	// decrement the packet counter indicate we are done with this packet
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON2, ECON2_PKTDEC);

	return (p);
}



/******************* (C) COPYRIGHT 2011 YEES *****END OF FILE****/




