/**
  ******************************************************************************
  * @file    spi_enc28j60.c
  * @author  YEES
  * @version V1.0.0
  * @date    2011/11/17
  * @brief   SPI functions ENC28J60. 
  *            
  *          +------------------------------------------+
  *          |             Pin assignment               |
  *          +-----------------------+------------------+
  *          |  STM32F4-Discovery    |     ENC28J60     |
  *          +-----------------------+------------------+
  *          | PB11                  |   RESET          |
  *          | PB12                  |   CS             |
  *          | PB13 / SPI2_SCLK      |   SCK            |
  *          | PB14 / SPI2_MISO      |   SO             |
  *          | PB15 / SPI2_MOSI      |   SI             |
  *          +-----------------------+------------------+
  ******************************************************************************
  * @attention
  * (C) COPYRIGHT 2011 YEES 
  ******************************************************************************  
  */ 

/* Includes ------------------------------------------------------------------*/
#include "spi_ENC28J60.h"

/** @addtogroup STM32F4xx_StdPeriph_Examples
  * @{
  */

/** @addtogroup SPI_ENC28J60
  * @{
  */  

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void spiENC28J60_LowLevel_DeInit(void);
void spiENC28J60_LowLevel_Init(void); 

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  DeInitializes the peripherals used by the SPI ENC28J60 driver.
  * @param  None
  * @retval None
  */
void spiENC28J60_DeInit(void)
{
  spiENC28J60_LowLevel_DeInit();
}

/**
  * @brief  Initializes the peripherals used by the SPI ENC28J60 driver.
  * @param  None
  * @retval None
  */
void spiENC28J60_Init(void)
{
#ifndef		USE_SOFTWAEW_SPI
  SPI_InitTypeDef  SPI_InitStructure;
#endif

  spiENC28J60_LowLevel_Init();
    
  /*!< Deselect the ENC28J60: Chip Select high */
  spiENC28J60_CS_HIGH();

#ifndef		USE_SOFTWAEW_SPI
  /*!< SPI configuration */
  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_128;

  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(spiENC28J60_SPI, &SPI_InitStructure);

  /*!< Enable the spiENC28J60_SPI  */
  SPI_Cmd(spiENC28J60_SPI, ENABLE);
#endif
}

/**
  * @brief  Reads a byte from the SPI Flash.
  * @note   This function must be used only if the Start_Read_Sequence function
  *         has been previously called.
  * @param  None
  * @retval Byte Read from the SPI Flash.
  */
uint8_t spiENC28J60_ReadByte(void)
{
  return (spiENC28J60_SendByte(sFLASH_DUMMY_BYTE));
}

/**
  * @brief  Sends a byte through the SPI interface and return the byte received
  *         from the SPI bus.
  * @param  byte: byte to send.
  * @retval The value of the received byte.
  */
uint8_t spiENC28J60_SendByte(uint8_t byte)
{
#ifndef		USE_SOFTWAEW_SPI
  /*!< Loop while DR register in not emplty */
  while (SPI_I2S_GetFlagStatus(spiENC28J60_SPI, SPI_I2S_FLAG_TXE) == RESET);

  /*!< Send byte through the SPI1 peripheral */
  SPI_I2S_SendData(spiENC28J60_SPI, byte);

  /*!< Wait to receive a byte */
  while (SPI_I2S_GetFlagStatus(spiENC28J60_SPI, SPI_I2S_FLAG_RXNE) == RESET);

  /*!< Return the byte read from the SPI bus */
  return SPI_I2S_ReceiveData(spiENC28J60_SPI);
#else
  uint16_t n;
  uint8_t in = 0x00;
  for(n=0;n<8;n++){
	spiENC28J60_SCK_LOW();
  	if(byte & (0x80 >> n)) spiENC28J60_MOSI_HIGH(); else spiENC28J60_MOSI_LOW();
	in <<= 1;
	if(spiENC28J60_MISO_GET()) in++;
	spiENC28J60_SCK_HIGH();
  }
  spiENC28J60_SCK_LOW();
  return(in);
#endif
}

/**
  * @brief  Initializes the peripherals used by the SPI ENC28J60 driver.
  * @param  None
  * @retval None
  */
void spiENC28J60_LowLevel_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

#ifndef		USE_SOFTWAEW_SPI
  /*!< Enable the SPI clock */
  spiENC28J60_SPI_CLK_INIT(spiENC28J60_SPI_CLK, ENABLE);
#endif

  /*!< Enable GPIO clocks */
  RCC_AHB1PeriphClockCmd(	spiENC28J60_SPI_SCK_GPIO_CLK 	| 
							spiENC28J60_SPI_MISO_GPIO_CLK 	| 
							spiENC28J60_SPI_MOSI_GPIO_CLK 	| 
							spiENC28J60_CS_GPIO_CLK 		| 
							spiENC28J60_RESET_GPIO_CLK, 		ENABLE);

  /*!< SPI pins configuration *************************************************/

#ifndef		USE_SOFTWAEW_SPI
  /*!< Connect SPI pins to AF5 */  
  GPIO_PinAFConfig(spiENC28J60_SPI_SCK_GPIO_PORT, spiENC28J60_SPI_SCK_SOURCE, spiENC28J60_SPI_SCK_AF);
  GPIO_PinAFConfig(spiENC28J60_SPI_MISO_GPIO_PORT, spiENC28J60_SPI_MISO_SOURCE, spiENC28J60_SPI_MISO_AF);
  GPIO_PinAFConfig(spiENC28J60_SPI_MOSI_GPIO_PORT, spiENC28J60_SPI_MOSI_SOURCE, spiENC28J60_SPI_MOSI_AF);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;

  /*!< SPI SCK pin configuration */
  GPIO_InitStructure.GPIO_Pin = spiENC28J60_SPI_SCK_PIN;
  GPIO_Init(spiENC28J60_SPI_SCK_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MOSI pin configuration */
  GPIO_InitStructure.GPIO_Pin =  spiENC28J60_SPI_MOSI_PIN;
  GPIO_Init(spiENC28J60_SPI_MOSI_GPIO_PORT, &GPIO_InitStructure);

  /*!< SPI MISO pin configuration */
  GPIO_InitStructure.GPIO_Pin =  spiENC28J60_SPI_MISO_PIN;
  GPIO_Init(spiENC28J60_SPI_MISO_GPIO_PORT, &GPIO_InitStructure);
#else
  /* spiENC28J60 SCK ******************************************/
  GPIO_InitStructure.GPIO_Pin = spiENC28J60_SPI_SCK_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(spiENC28J60_SPI_SCK_GPIO_PORT, &GPIO_InitStructure);
  
  /* spiENC28J60 MOSI ******************************************/
  GPIO_InitStructure.GPIO_Pin = spiENC28J60_SPI_MOSI_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(spiENC28J60_SPI_MOSI_GPIO_PORT, &GPIO_InitStructure);
  
  /* spiENC28J60 MISO ******************************************/
  GPIO_InitStructure.GPIO_Pin = spiENC28J60_SPI_MISO_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(spiENC28J60_SPI_MISO_GPIO_PORT, &GPIO_InitStructure);
#endif        
  /* spiENC28J60 CS ******************************************/
  GPIO_InitStructure.GPIO_Pin = spiENC28J60_CS_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(spiENC28J60_CS_GPIO_PORT, &GPIO_InitStructure);
  
  /* spiENC28J60 RESET ******************************************/
  GPIO_InitStructure.GPIO_Pin = spiENC28J60_RESET_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(spiENC28J60_RESET_GPIO_PORT, &GPIO_InitStructure);
  
  spiENC28J60_CS_HIGH();
  spiENC28J60_RESET_HIGH();
}

/**
  * @brief  DeInitializes the peripherals used by the SPI ENC28J60 driver.
  * @param  None
  * @retval None
  */
void spiENC28J60_LowLevel_DeInit(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

#ifndef		USE_SOFTWAEW_SPI
  /*!< Disable the spiENC28J60_SPI  ************************************************/
  SPI_Cmd(spiENC28J60_SPI, DISABLE);

  /*!< DeInitializes the spiENC28J60_SPI *******************************************/
  SPI_I2S_DeInit(spiENC28J60_SPI);
  
  /*!< spiENC28J60_SPI Periph clock disable ****************************************/
  spiENC28J60_SPI_CLK_INIT(spiENC28J60_SPI_CLK, DISABLE);
#endif
      
  /*!< Configure all pins used by the SPI as input floating *******************/
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;

  GPIO_InitStructure.GPIO_Pin = spiENC28J60_SPI_SCK_PIN;
  GPIO_Init(spiENC28J60_SPI_SCK_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = spiENC28J60_SPI_MISO_PIN;
  GPIO_Init(spiENC28J60_SPI_MISO_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = spiENC28J60_SPI_MOSI_PIN;
  GPIO_Init(spiENC28J60_SPI_MOSI_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = spiENC28J60_CS_PIN;
  GPIO_Init(spiENC28J60_CS_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = spiENC28J60_RESET_PIN;
  GPIO_Init(spiENC28J60_RESET_GPIO_PORT, &GPIO_InitStructure);
}

/**
  * @}
  */

/**
  * @}
  */

/******************* (C) COPYRIGHT 2011 YEES *****END OF FILE****/
